@@ -19,6 +19,8 @@ void InitPrecPriorMEM(BEAST2_MODEL_PTR model,BEAST2_OPTIONS_PTR opt,MemPointers
 		MODEL.nPrec=1L;    
 		MODEL.precVal=opt->prior.precValue;	
 		MODEL.logPrecVal=logf(MODEL.precVal);
+		MODEL.precVec=&(MODEL.precVal);
+		MODEL.logPrecVec=&(MODEL.logPrecVal);
 		MODEL.curr.precXtXDiag=&MODEL.precVal; 
 		MODEL.prop.precXtXDiag=&MODEL.precVal;
 		for (int i=0; i < MODEL.NUMBASIS; i++) {
@@ -37,15 +39,15 @@ void InitPrecPriorMEM(BEAST2_MODEL_PTR model,BEAST2_OPTIONS_PTR opt,MemPointers
 		r_ippsSet_32f(precValue,MODEL.precVec,MODEL.nPrec);
 		r_ippsSet_32f(logf(precValue),MODEL.logPrecVec,MODEL.nPrec);
 		I32 K_MAX=opt->prior.K_MAX;
-		MODEL.curr.precXtXDiag=MyALLOC(*MEM,K_MAX,F32,64);
-		MODEL.prop.precXtXDiag=MyALLOC(*MEM,K_MAX,F32,64);
+		MODEL.curr.precXtXDiag=MyALLOC(*MEM,K_MAX * 2,F32,64);
+		MODEL.prop.precXtXDiag=MODEL.curr.precXtXDiag+K_MAX;
 		for (int i=0; i < MODEL.NUMBASIS; i++) {
 			BEAST2_BASIS_PTR b=MODEL.b+i;
 			b->nPrec=-9999L;
 			b->offsetPrec=-9999L;
 		}
-		MODEL.curr.nTermsPerPrecGrp=NULL;
-		MODEL.prop.nTermsPerPrecGrp=NULL;
+		MODEL.curr.nTermsPerPrecGrp=MyALLOC(*MEM,MODEL.nPrec * 2,I16,0);
+		MODEL.prop.nTermsPerPrecGrp=MODEL.curr.nTermsPerPrecGrp+MODEL.nPrec;
 	}
  	else if (precType==OrderWise)
 	{
@@ -66,22 +68,28 @@ void InitPrecPriorMEM(BEAST2_MODEL_PTR model,BEAST2_OPTIONS_PTR opt,MemPointers
 		F32 precValue=opt->prior.precValue;
 		r_ippsSet_32f(precValue,MODEL.precVec,MODEL.nPrec);
 		r_ippsSet_32f(logf(precValue),MODEL.logPrecVec,MODEL.nPrec);
-		MODEL.curr.nTermsPerPrecGrp=MyALLOC(*MEM,MODEL.nPrec * 2,I08,64); 	
+		MODEL.curr.nTermsPerPrecGrp=MyALLOC(*MEM,MODEL.nPrec * 2,I16,64); 	
 		MODEL.prop.nTermsPerPrecGrp=MODEL.curr.nTermsPerPrecGrp+MODEL.nPrec;
 		I32 K_MAX=opt->prior.K_MAX;
 		MODEL.curr.precXtXDiag=MyALLOC(*MEM,K_MAX * 2,F32,64);
 		MODEL.prop.precXtXDiag=MODEL.curr.precXtXDiag+K_MAX;
 	}
 }
-void GetNumTermsPerPrecGrp_prec012(BEAST2_MODEL_PTR model) {
+void GetNumTermsPerPrecGrp_prec01(BEAST2_MODEL_PTR model) {
+	return;
+}
+void GetNumTermsPerPrecGrp_prec2(BEAST2_MODEL_PTR model) {
+	for (int i=0; i < model->NUMBASIS; i++) {
+		model->curr.nTermsPerPrecGrp[i]=model->b[i].K;
+	}
 	return;
 }
 void GetNumTermsPerPrecGrp_prec3(BEAST2_MODEL_PTR  model) { 
-	memset(MODEL.curr.nTermsPerPrecGrp,0,MODEL.nPrec);
-	for (int id=0; id < MODEL.NUMBASIS; id++) {
-		I08PTR   nTermsPerGrp=MODEL.curr.nTermsPerPrecGrp+MODEL.b[id].offsetPrec;
+	memset(MODEL.curr.nTermsPerPrecGrp,0,MODEL.nPrec *sizeof(I16));
+	for (int id=0; id < MODEL.NUMBASIS;++id) {
+		I16PTR   nTermsPerGrp=MODEL.curr.nTermsPerPrecGrp+MODEL.b[id].offsetPrec;
 		U08PTR   termType=MODEL.b[id].termType;
-		for (int k=0; k < MODEL.b[id].K; k++) {	 
+		for (int k=0; k < MODEL.b[id].K;++k) {	 
 			nTermsPerGrp[ termType[k]-1 ]++;
 		}
 	}
@@ -89,43 +97,45 @@ void GetNumTermsPerPrecGrp_prec3(BEAST2_MODEL_PTR  model) {
 void GetXtXPrecDiag_prec01(BEAST2_MODEL_PTR model) {return;}
 void GetXtXPrecDiag_prec2(BEAST2_MODEL_PTR model) {
 	F32PTR precXtXDiag=MODEL.curr.precXtXDiag;
-	for (int id=0; id < MODEL.NUMBASIS; id++) {
+	for (int id=0; id < MODEL.NUMBASIS;++id) {
 		F32		  prec=MODEL.precVec[id];  
-		for (int k=0; k < MODEL.b[id].K; k++) {
+		for (int k=0; k < MODEL.b[id].K;++k) {
 			*precXtXDiag++=prec;
 		}
 	}
 }
 void GetXtXPrecDiag_prec3( BEAST2_MODEL_PTR model) {
 	F32PTR precXtXDiag=MODEL.curr.precXtXDiag;
-	for (int id=0; id < MODEL.NUMBASIS; id++) {
+	for (int id=0; id < MODEL.NUMBASIS;++id) {
 		U08PTR    termType=MODEL.b[id].termType;
 		F32PTR    prec=MODEL.precVec+MODEL.b[id].offsetPrec;
-		for (int k=0; k < MODEL.b[id].K; k++) {
+		for (int k=0; k < MODEL.b[id].K;++k) {
 			*precXtXDiag++=prec[termType[k] - 1];
 		}
 	} 
 }
-void UpdateXtXPrec_nTermsPerGrp_prec01(BEAST2_MODEL_PTR model,BEAST2_BASIS_PTR basis,NEWTERM_PTR new){
+void UpdateXtXPrec_nTermsPerGrp_prec01(BEAST2_MODEL_PTR model,BEAST2_BASIS_PTR basis,NEWTERM_PTR new,NEWCOLINFO_PTR newcol){
 	return; 
 }
-void UpdateXtXPrec_nTermsPerGrp_prec2(BEAST2_MODEL_PTR model,BEAST2_BASIS_PTR basis,NEWTERM_PTR new) {
+void UpdateXtXPrec_nTermsPerGrp_prec2(BEAST2_MODEL_PTR model,BEAST2_BASIS_PTR basis,NEWTERM_PTR new,NEWCOLINFO_PTR newcol) {
 	I32 Kold=MODEL.curr.K;
-	SCPY(new->k1 - 1,MODEL.curr.precXtXDiag,MODEL.prop.precXtXDiag);
-	SCPY(Kold - new->k2_old,MODEL.curr.precXtXDiag+new->k2_old,MODEL.prop.precXtXDiag+new->k2_new);
-	I32 idx=basis - model->b;
-	F32 prec=MODEL.precVec[idx];
+	SCPY(newcol->k1 - 1,MODEL.curr.precXtXDiag,MODEL.prop.precXtXDiag);
+	SCPY(Kold- newcol->k2_old,MODEL.curr.precXtXDiag+newcol->k2_old,MODEL.prop.precXtXDiag+newcol->k2_new);
+	I32 id=basis - model->b;
+	F32 prec=MODEL.precVec[id];
 	F32PTR  precXtXDiag=MODEL.prop.precXtXDiag;
-	for (int k=new->k1 ; k < new->k2_new; k++) {
+	for (int k=newcol->k1 ; k <=newcol->k2_new;++k) {
 		precXtXDiag[k - 1]=prec;		
 	}
+	memcpy(MODEL.prop.nTermsPerPrecGrp,MODEL.curr.nTermsPerPrecGrp,sizeof(I16) * model->NUMBASIS);
+	MODEL.prop.nTermsPerPrecGrp[id]=MODEL.curr.nTermsPerPrecGrp[id]+newcol->k2_new - newcol->k2_old;
 }
-void UpdateXtXPrec_nTermsPerGrp_prec3(BEAST2_MODEL_PTR model,BEAST2_BASIS_PTR basis,NEWTERM_PTR new)
+void UpdateXtXPrec_nTermsPerGrp_prec3(BEAST2_MODEL_PTR model,BEAST2_BASIS_PTR basis,NEWTERM_PTR new,NEWCOLINFO_PTR newcol)
 {
 	I32 Kold=MODEL.curr.K;
-	SCPY(new->k1 - 1,MODEL.curr.precXtXDiag,MODEL.prop.precXtXDiag);
-	SCPY(Kold - new->k2_old,MODEL.curr.precXtXDiag+new->k2_old,MODEL.prop.precXtXDiag+new->k2_new);
-	F32PTR  precXtXDiag=MODEL.prop.precXtXDiag+new->k1 - 1;
+	SCPY(newcol->k1 - 1,MODEL.curr.precXtXDiag,MODEL.prop.precXtXDiag);
+	SCPY(Kold - newcol->k2_old,MODEL.curr.precXtXDiag+newcol->k2_old,MODEL.prop.precXtXDiag+newcol->k2_new);
+	F32PTR  precXtXDiag=MODEL.prop.precXtXDiag+newcol->k1 - 1;
 	F32PTR  prec=MODEL.precVec+basis->offsetPrec;
 	if (basis->type==SEASONID){
 		for (int i=0; i < new->numSeg; i++) {
@@ -148,40 +158,37 @@ void UpdateXtXPrec_nTermsPerGrp_prec3(BEAST2_MODEL_PTR model,BEAST2_BASIS_PTR ba
 	}
 	#undef  NEW   
 	#define NEW (*new)
-	memcpy(MODEL.prop.nTermsPerPrecGrp,MODEL.curr.nTermsPerPrecGrp,MODEL.nPrec);
-	I32 k1=NEW.k1     - basis->Kbase;
-	I32 k2old=NEW.k2_old - basis->Kbase;
-	I32 k2new=NEW.k2_new - basis->Kbase;
-	I32 nPrec=basis->nPrec;
-	U08PTR termType=basis->termType;
-	I08PTR numTerms=MODEL.prop.nTermsPerPrecGrp+basis->offsetPrec;
-	memset(numTerms,0,nPrec);
+	I16PTR nTermsPerPrecGrp=MODEL.prop.nTermsPerPrecGrp+basis->offsetPrec;
+	memcpy(MODEL.prop.nTermsPerPrecGrp,MODEL.curr.nTermsPerPrecGrp,sizeof(I16) * MODEL.nPrec); 	
+	memset(nTermsPerPrecGrp,0,sizeof(I16) * basis->nPrec);
 	if (basis->type==SEASONID||basis->type==TRENDID) {
+		I32 k1=newcol->k1     - basis->Kbase;
+		I32 k2old=newcol->k2_old - basis->Kbase;
+		I32 k2new=newcol->k2_new - basis->Kbase;
+		U08PTR termType=basis->termType;
 		for (int i=1; i <=k1 - 1; i++) {
-			numTerms[termType[i - 1] - 1]++;
-		}
-		I32 Kbasis_old=basis->K;
-		for (int i=k2old+1; i <=Kbasis_old; i++) {
-			numTerms[termType[i - 1] - 1]++;
+			nTermsPerPrecGrp[termType[i - 1] - 1]++;
 		}
 		if (basis->type==SEASONID) {			
 			for (int i=0; i < NEW.numSeg; i++) {
 				for (int order=new->SEG[i].ORDER1; order <=new->SEG[i].ORDER2; order++) {
-					numTerms[order - 1]+=2;
+					nTermsPerPrecGrp[order - 1]+=2;
+				}
+			}
+		} else		{
+			for (int i=0; i < NEW.numSeg; i++) {
+				for (int order=new->SEG[i].ORDER1; order <=new->SEG[i].ORDER2; order++) {
+					nTermsPerPrecGrp[order ]+=1;
 				}
 			}
 		}
-		else
-		{
-			for (int i=0; i < NEW.numSeg; i++) {
-				for (int order=new->SEG[i].ORDER1; order <=new->SEG[i].ORDER2; order++) {
-					numTerms[order ]+=1;
-				}
-			}
+		I32 Kbasis_old=basis->K;
+		for (int i=k2old+1; i <=Kbasis_old; i++) {
+			nTermsPerPrecGrp[termType[i - 1] - 1]++;
 		}
 	}
 	else if (basis->type==OUTLIERID) {
-		numTerms[0]=NEW.nKnot_new;
+		nTermsPerPrecGrp[0]=NEW.nKnot_new;
 	}
 #undef  NEW
 }
@@ -191,10 +198,10 @@ void ResamplePrecValues_prec0(BEAST2_MODEL_PTR model,BEAST2_HyperPar*hyperPar,VO
 void ResamplePrecValues_prec1(BEAST2_MODEL_PTR model,BEAST2_HyperPar *hyperPar,VOID_PTR stream) {
 	I32 K=MODEL.curr.K;
 	F32 sumq=DOT(K,MODEL.beta,MODEL.beta);
-	r_vsRngGamma(VSL_RNG_METHOD_GAMMA_GNORM_ACCURATE,(*(VSLStreamStatePtr*)stream),1L,&(model->precVal),(hyperPar->del_1+K * 0.5f),0,1.f);
-	F32 newPrecVal=model->precVal/(hyperPar->del_2+0.5f * sumq/model->sig2);
-	model->precVal=newPrecVal > MIN_PREC_VALUE? newPrecVal: model->precVal;
-	model->logPrecVal=logf(model->precVal);
+	r_vsRngGamma(VSL_RNG_METHOD_GAMMA_GNORM_ACCURATE,(*(VSLStreamStatePtr*)stream),1L,model->precVec,(hyperPar->del_1+K * 0.5f),0,1.f);
+	F32 newPrecVal=model->precVec[0]/(hyperPar->del_2+0.5f * sumq/model->sig2);
+	model->precVec[0]=newPrecVal > MIN_PREC_VALUE? newPrecVal: MIN_PREC_VALUE;
+	model->logPrecVec[0]=logf(model->precVec[0]);
 }
 void ResamplePrecValues_prec2(BEAST2_MODEL_PTR model,BEAST2_HyperPar *hyperPar,VOID_PTR stream) {
 		for (int id=0; id < MODEL.NUMBASIS; id++) {
@@ -205,9 +212,9 @@ void ResamplePrecValues_prec2(BEAST2_MODEL_PTR model,BEAST2_HyperPar *hyperPar,V
 			F32		sumq=DOT(K,beta,beta);
 			r_vsRngGamma(VSL_RNG_METHOD_GAMMA_GNORM_ACCURATE,(*(VSLStreamStatePtr*)stream),1,&(MODEL.precVec[id]),(hyperPar->del_1+K * 0.5f),0,1.f);
 			F32 newPrecVal=MODEL.precVec[id]/(hyperPar->del_2+0.5f * sumq/MODEL.sig2);
-			MODEL.precVec[id]=newPrecVal > MIN_PREC_VALUE ? newPrecVal : MODEL.precVec[id];
-			MODEL.logPrecVec[id]=logf(MODEL.precVec[id]);
-		}
+			MODEL.precVec[id]=newPrecVal > MIN_PREC_VALUE ? newPrecVal : MIN_PREC_VALUE;
+			MODEL.logPrecVec[id]=logf(MODEL.precVec[id]);	
+		}		
 }
 void ResamplePrecValues_prec3(BEAST2_MODEL_PTR model,BEAST2_HyperPar *hyperPar,VOID_PTR stream) {
 	for (int id=0; id < MODEL.NUMBASIS; id++) {
@@ -227,7 +234,7 @@ void ResamplePrecValues_prec3(BEAST2_MODEL_PTR model,BEAST2_HyperPar *hyperPar,V
 			if (K > 0) {
 				r_vsRngGamma(VSL_RNG_METHOD_GAMMA_GNORM_ACCURATE,(*(VSLStreamStatePtr*)stream),1,&prec[i - 1],(hyperPar->del_1+K * 0.5f),0,1.f);
 				F32 newPrecVal=prec[i - 1]/(hyperPar->del_2+0.5f * sumq/MODEL.sig2);
-				prec[i - 1]=newPrecVal > MIN_PREC_VALUE ? newPrecVal : prec[i - 1];
+				prec[i - 1]=newPrecVal > MIN_PREC_VALUE ? newPrecVal : MIN_PREC_VALUE;
 				logPrec[i - 1]=logf(prec[i - 1]);
 			}
 		}
@@ -237,8 +244,8 @@ void IncreasePrecValues_prec0(BEAST2_MODEL_PTR model) {
 	return;
 }
 void IncreasePrecValues_prec1(BEAST2_MODEL_PTR model) {
-	model->precVal=model->precVal+model->precVal;
-	model->logPrecVal=logf(model->precVal);
+	model->precVec[0]=model->precVec[0]+model->precVec[0];
+	model->logPrecVec[0]=logf(model->precVec[0]);
 }
 void IncreasePrecValues_prec2(BEAST2_MODEL_PTR model) {
 	for (int id=0; id < MODEL.NUMBASIS;++id) {
@@ -252,13 +259,10 @@ void IncreasePrecValues_prec3(BEAST2_MODEL_PTR model) {
 		F32PTR prec=MODEL.precVec+basis->offsetPrec;;
 		F32PTR logPrec=MODEL.logPrecVec+basis->offsetPrec;;
 		U08PTR termType=basis->termType;
-		F32PTR beta=MODEL.beta+basis->Kbase;
 		for (int i=1; i <=basis->nPrec; i++) {
 			I32 K=0;
 			for (int j=0; j < basis->K; j++) {
-				if (termType[j]==i) {
-					K++;
-				}
+				if (termType[j]==i) { K++; }
 			}
 			if (K > 0) {								
 				prec[i - 1]=prec[i - 1]+prec[i - 1];
@@ -279,21 +283,7 @@ void ComputeMargLik_prec01(BEAST2_MODELDATA_PTR data,BEAST2_MODEL_PTR model,BEAS
 	data->alpha2_star=alpha2_star;
 	data->marg_lik=marg_lik;
 }
-void ComputeMargLik_prec2(BEAST2_MODELDATA_PTR data,BEAST2_MODEL_PTR model,BEAST2_YINFO_PTR yInfo,
-								BEAST2_HyperPar_PTR hyperPar) {
-	I32 K=data->K;
-	solve_U_as_LU_invdiag_sqrmat(data->cholXtX,data->XtY,data->beta_mean,K);	
-	F32 alpha2_star=( yInfo->YtY_plus_alpha2Q[0] - DOT(K,data->XtY,data->beta_mean) ) * 0.5;
-	F32 half_log_det_post=sum_log_diagv2(data->cholXtX,K);
-	F32 half_log_det_prior=data->precXtXDiag[0]*model->b[0]. K;
-	if (model->NUMBASIS==2) 	half_log_det_prior+=data->precXtXDiag[1] * model->b[1].K;		
-	else						half_log_det_prior+=data->precXtXDiag[1] * model->b[1].K+data->precXtXDiag[2] * model->b[2].K;
-	half_log_det_prior=-0.5 *half_log_det_prior;
-	F32 marg_lik=half_log_det_post- half_log_det_prior- yInfo->alpha1_star* fastlog(alpha2_star);
-	data->alpha2_star=alpha2_star;
-	data->marg_lik=marg_lik;
-}
-void ComputeMargLik_prec3(BEAST2_MODELDATA_PTR data,BEAST2_MODEL_PTR model,BEAST2_YINFO_PTR yInfo, 
+void ComputeMargLik_prec23(BEAST2_MODELDATA_PTR data,BEAST2_MODEL_PTR model,BEAST2_YINFO_PTR yInfo, 
 	BEAST2_HyperPar_PTR hyperPar) {
 	I32 K=data->K;
 	solve_U_as_LU_invdiag_sqrmat(data->cholXtX,data->XtY,data->beta_mean,K);
@@ -327,14 +317,14 @@ void SetUpPrecFunctions(I08 precPriorType,I32 q,PREC_FUNCS * funcs) {
 	if (q==1) {
 			if (  precPriorType==ConstPrec||precPriorType==UniformPrec) {
 				funcs->GetXtXPrecDiag=GetXtXPrecDiag_prec01;
-				funcs->GetNumTermsPerPrecGrp=GetNumTermsPerPrecGrp_prec012;
+				funcs->GetNumTermsPerPrecGrp=GetNumTermsPerPrecGrp_prec01;
 				funcs->UpdateXtXPrec_nTermsPerGrp=UpdateXtXPrec_nTermsPerGrp_prec01;
 				funcs->chol_addCol=chol_addCol_skipleadingzeros_prec_invdiag;
 			}
 			else if (precPriorType==ComponentWise)
 			{
 				funcs->GetXtXPrecDiag=GetXtXPrecDiag_prec2;
-				funcs->GetNumTermsPerPrecGrp=GetNumTermsPerPrecGrp_prec012;
+				funcs->GetNumTermsPerPrecGrp=GetNumTermsPerPrecGrp_prec2;
 				funcs->UpdateXtXPrec_nTermsPerGrp=UpdateXtXPrec_nTermsPerGrp_prec2;
 				funcs->chol_addCol=chol_addCol_skipleadingzeros_precVec_invdiag;
 			}
@@ -355,20 +345,20 @@ void SetUpPrecFunctions(I08 precPriorType,I32 q,PREC_FUNCS * funcs) {
 			else if (precPriorType==3)	funcs->ResamplePrecValues=ResamplePrecValues_prec3;
 			if (precPriorType==0)	        funcs->ComputeMargLik=ComputeMargLik_prec01;
 			else if (precPriorType==1) 	funcs->ComputeMargLik=ComputeMargLik_prec01;
-			else if (precPriorType==2)	funcs->ComputeMargLik=ComputeMargLik_prec2;
-			else if (precPriorType==3)	funcs->ComputeMargLik=ComputeMargLik_prec3;
+			else if (precPriorType==2)	funcs->ComputeMargLik=ComputeMargLik_prec23;
+			else if (precPriorType==3)	funcs->ComputeMargLik=ComputeMargLik_prec23;
 	}
 	if (q> 1) {
 			if (  precPriorType==ConstPrec||precPriorType==UniformPrec) {
 				funcs->GetXtXPrecDiag=GetXtXPrecDiag_prec01;
-				funcs->GetNumTermsPerPrecGrp=GetNumTermsPerPrecGrp_prec012;
+				funcs->GetNumTermsPerPrecGrp=GetNumTermsPerPrecGrp_prec01;
 				funcs->UpdateXtXPrec_nTermsPerGrp=UpdateXtXPrec_nTermsPerGrp_prec01;
 				funcs->chol_addCol=chol_addCol_skipleadingzeros_prec_invdiag;
 			}
 			else if (precPriorType==ComponentWise)
 			{
 				funcs->GetXtXPrecDiag=GetXtXPrecDiag_prec2;
-				funcs->GetNumTermsPerPrecGrp=GetNumTermsPerPrecGrp_prec012;
+				funcs->GetNumTermsPerPrecGrp=GetNumTermsPerPrecGrp_prec2;
 				funcs->UpdateXtXPrec_nTermsPerGrp=UpdateXtXPrec_nTermsPerGrp_prec2;
 				funcs->chol_addCol=chol_addCol_skipleadingzeros_precVec_invdiag;
 			}
